/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.techfak.scie.classifiers;

import de.unibi.techfak.scie.classifiers.data.ClassificationResult;
import de.unibi.techfak.scie.classifiers.data.DataPoint;
import de.unibi.techfak.scie.classifiers.data.FeatureDictionary;
import de.unibi.techfak.scie.classifiers.data.LabeledDataPoint;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;

/**
 * This is an interface for classifiers that are able to correctly treat our
 * data structure.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public interface Classifier {

	/**
	 * Adjust the classifier parameters according to the given training data.
	 *
	 * @param trainingData a list of LabeledDataPoints. The user has to assure
	 * that the training data points stem from a consistent data set.
	 */
	public void train(ArrayList<LabeledDataPoint> trainingData);

	/**
	 * Classify the given DataPoint.
	 *
	 * @param data a DataPoint.
	 * @return a ClassificationResult for that DataPoint.
	 */
	public ClassificationResult classify(DataPoint data);

	/**
	 * Write the current parameters and features of the classifier to the given
	 * Writer.
	 *
	 * @param parameterOut a Writer for the parameters.
	 * @param featuresOut a Writer for the features.
	 * @throws java.io.IOException
	 */
	public void writeParamters(Writer parameterOut, Writer featuresOut) throws IOException;

	/**
	 * Override the current parameters with the ones given in the Reader.
	 *
	 * @param parameterIn a Reader containing parameter data.
	 * @param featuresIn a Reader containing the fitting feature data.
	 * @throws java.io.IOException
	 */
	public void readParameters(Reader parameterIn, Reader featuresIn) throws IOException;

	/**
	 * Returns the FeatureDictionary for this Classifier.
	 *
	 * @return the FeatureDictionary for this Classifier.
	 */
	public FeatureDictionary getFeatureDictionary();

}
