/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.techfak.scie.classifiers.data;

import java.util.Objects;

/**
 * This is an interface container class for the result of a classifier for a
 * given DataPoint. It stores the DataPoint itself, the label given by the
 * classifier and the Classifier confidence, meaning a probability that the
 * given data point should have a positive label.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ClassificationResult {

	private final DataPoint data;
	private final boolean label;
	private final double confidence;

	public ClassificationResult(DataPoint data, boolean label, double confidence) {
		this.data = data;
		this.label = label;
		this.confidence = confidence;
	}

	/**
	 * Returns the confidence that this results DataPoint should have a
	 * positive label.
	 *
	 * @return a confidence between 0 and 1, 1 being absolute confidence in a
	 * "true" label and 0 being absolute confidence in a "false" label.
	 */
	public double getConfidence() {
		return confidence;
	}

	public DataPoint getData() {
		return data;
	}

	public boolean getLabel() {
		return label;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 61 * hash + Objects.hashCode(this.data);
		hash = 61 * hash + (this.label ? 1 : 0);
		hash = 61 * hash + (int) (Double.doubleToLongBits(this.confidence) ^ (Double.doubleToLongBits(this.confidence) >>> 32));
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final ClassificationResult other = (ClassificationResult) obj;
		if (!Objects.equals(this.data, other.data)) {
			return false;
		}
		if (this.label != other.label) {
			return false;
		}
		if (Double.doubleToLongBits(this.confidence) != Double.doubleToLongBits(other.confidence)) {
			return false;
		}
		return true;
	}

}
