/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.classifiers.data;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Map.Entry;
import java.util.Objects;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This dictionary keeps track of which features actually exist in a certain
 * dataset and thus enables us to address features with index notation and name
 * notation alike.
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class FeatureDictionary {

    private final TreeMap<String, Integer> indexMapping = new TreeMap<>();
    private final TreeMap<Integer, String> nameMapping = new TreeMap<>();
    private int maxIdx = 0;

    public FeatureDictionary() {
    }

    public void addFeature(final String name) {
        if (!indexMapping.containsKey(name)) {
            maxIdx++;
            indexMapping.put(name, maxIdx);
            nameMapping.put(maxIdx, name);
        }
    }

    public boolean hasFeature(final String name) {
        return indexMapping.containsKey(name);
    }

    /**
     * Returns a unique index for the feature with the given name or -1 if no
     * such feature exists.
     *
     * @param name
     *
     * @return
     */
    public int getFeatureIndex(final String name) {
        final Integer index = indexMapping.get(name);
        if (index == null) {
            return -1;
        }
        return index;
    }

    /**
     * Returns the name for the feature with the given index or null if no such
     * feature is known.
     *
     * @param idx
     *
     * @return
     */
    public String getFeatureName(final int idx) {
        return nameMapping.get(idx);
    }

    /**
     * Removes the feature with the given name and returns its former index. If
     * the feature was not known in the FeatureDictionary, -1 is returned.
     *
     * @param name the features name.
     * @return the former feature index.
     */
    public int removeFeature(final String name) {
        final Integer index = indexMapping.remove(name);
        if (index != null) {
            nameMapping.remove(index);
            if (index == maxIdx) {
                maxIdx = nameMapping.lastKey();
            }
            return index;
        } else {
            return -1;
        }
    }

    /**
     * Removes the feature with the given index and returns its former name. If
     * no feature with that index exists, null is returned.
     *
     * @param index the features index.
     * @return the former feature name.
     */
    public String removeFeature(final int index) {
        final String name = nameMapping.remove(index);
        if (name != null) {
            indexMapping.remove(name);
            if (index == maxIdx) {
                if (!nameMapping.isEmpty()) {
                    maxIdx = nameMapping.lastKey();
                } else {
                    maxIdx = 0;
                }
            }
        }
        return name;
    }

    public int getNumberOfFeatures() {
        return indexMapping.size();
    }

    /**
     * It is not recommended to work on this datastructure directly! Please use
     * this only for reading purposes!
     *
     * @return
     */
    public SortedMap<String, Integer> getIndexMapping() {
        return indexMapping;
    }

    /**
     * It is not recommended to work on this datastructure directly! Please use
     * this only for reading purposes!
     *
     * @return
     */
    public SortedMap<Integer, String> getNameMapping() {
        return nameMapping;
    }

    /**
     * Stores this FeatureDictionary to the given Writer.
     *
     * @param out a Writer.
     */
    public void store(Writer out) throws IOException {
        for (final Entry<Integer, String> entry : nameMapping.entrySet()) {
            out.write(entry.getKey() + ":\"" + entry.getValue() + "\"");
            if (entry.getKey() < maxIdx) {
                out.write("\n");
            }
        }
    }

    private static final Pattern LINE = Pattern.compile("^(\\d+)\\:\"(.*)\"$");

    /**
     * Loads the content from the given reader to fill this FeatureDictionary.
     *
     * @param in a Reader.
     */
    public void load(Reader in) throws IOException {
        indexMapping.clear();
        nameMapping.clear();
        final BufferedReader bufIn;
        if (in instanceof BufferedReader) {
            bufIn = (BufferedReader) in;
        } else {
            bufIn = new BufferedReader(in);
        }
        String line;
        while ((line = bufIn.readLine()) != null) {
            final Matcher matcher = LINE.matcher(line);
            if (matcher.matches()) {
                final int f = Integer.parseInt(matcher.group(1));
                if (nameMapping.containsKey(f)) {
                    throw new UnsupportedOperationException("The feature index " + f + " was used twice!");
                }
                final String featureName = matcher.group(2);
                indexMapping.put(featureName, f);
                nameMapping.put(f, featureName);
                if (f > maxIdx) {
                    maxIdx = f;
                }
            } else {
                throw new UnsupportedOperationException("The given input line: "
                        + line + " could not be read!");
            }
        }
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 47 * hash + Objects.hashCode(this.nameMapping);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final FeatureDictionary other = (FeatureDictionary) obj;
        if (!Objects.equals(this.nameMapping, other.nameMapping)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        try {
            final StringWriter out = new StringWriter();
            store(out);
            out.close();
            return out.toString();
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }
}
