/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.techfak.scie.classifiers;

import de.bwaldvogel.liblinear.SolverType;
import de.unibi.techfak.scie.classifiers.ClassifierEvaluation.SetEvaluation;
import de.unibi.techfak.scie.classifiers.data.DataPoint;
import de.unibi.techfak.scie.classifiers.data.FeatureDictionary;
import de.unibi.techfak.scie.classifiers.data.FeatureMap;
import de.unibi.techfak.scie.classifiers.data.LabeledDataPoint;
import java.util.ArrayList;
import java.util.Random;
import junit.framework.TestCase;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class LibLinearClassifierTest extends TestCase {

    public LibLinearClassifierTest(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * Test of train method, of class LibLinearClassifier.
     */
    public void testTrain() {
        /*
         * The idea is to create a perfectly seperable data set that can be
         * seperated if two dimensions only are considered. All other dimensions
         * are irrelevant. In that sense this is a good test for the L1
         * regularization.
         *
         * This serves as a three-fold-test:
         * <ol><li>Does LibLinear finds a perfect seperation?</li>
         * <li>Does it find the correct relevant features?</li>
         * <li>Does the evaluation make sense?</li></ol>
         */
        final int N = 1000;
        final int D = 300;
        final double margin = 1;
        //first we create the data set
        final Random rand = new Random();
        final ArrayList<LabeledDataPoint> data = new ArrayList<>();
        for (int n = 0; n < N; n++) {
            final double[] vec = new double[D];
            //we choose the label uniformly randomly
            final boolean label = rand.nextDouble() > 0.5;
            //we create the first coordinate normally distributed around 0.
            vec[0] = rand.nextGaussian() * 10;
            //the second coordinate depends on x and the label:
            //positive labelled data is above the line y = x and negative
            //labelled data is below the line y = x
            vec[1] = rand.nextGaussian() * 0.3 + vec[0];
            if (label) {
                vec[1] += margin;
            } else {
                vec[1] -= margin;
            }
            //the data is perfectly seperable now already. But now we add
            //noisy dimensions.
            for (int d = 2; d < D; d++) {
                vec[d] = rand.nextDouble() * 10;
            }
            data.add(new LabeledDataPoint(label, new VectorDataPoint(vec)));
        }
        //then train a classifier.
        final LibLinearClassifier instance = new LibLinearClassifier(SolverType.L1R_LR);
        instance.setC(0.01);
        instance.setEpsilon(0.01);
        instance.train(data);
        //First check if the basic numbers are plausible.
        assertEquals(2, instance.getTrainedModel().getNrClass());
        assertEquals(D - 1, instance.getTrainedModel().getNrFeature());
        assertTrue(instance.getTrainedModel().isProbabilityModel());
        //and evaluate it.
        final SetEvaluation eval = new SetEvaluation(instance, data, 21, false);
        //check the results, which we expect to be near perfect.
        assertEquals(N, eval.getN());
        assertEquals(1, eval.getAccuracy(), 0.1);
        assertEquals(1, eval.getPrecision(), 0.1);
        assertEquals(1, eval.getRecall(), 0.1);
        assertEquals(1, eval.getF1(), 0.1);
        assertEquals(1, eval.getArea_under_roc(), 0.1);
        //also check the number of non-zero-features, which should be 2.
        assertEquals(2, instance.getFeatureDictionary().getNumberOfFeatures(), 5);
    }

    public static final class VectorDataPoint implements DataPoint {

        private final double[] data;

        public VectorDataPoint(double[] data) {
            this.data = data;
        }

        @Override
        public FeatureMap getFeatureRepresentation(FeatureDictionary dict, boolean addFeaturesToDictionary) {
            final FeatureMap features = new FeatureMap(dict);
            for (int d = 0; d < data.length; d++) {
                features.addNumericFeature("dim" + d, data[d],
                        addFeaturesToDictionary);
            }
            return features;
        }

    }

}
